<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Ems\Repository\ProcessoRepository;
use Ems\Util\ExpectedException;
use Ems\Util\StringUtil;
use Ems\Util\Utils;
use Geral\Util\NovoCodigo;
use Util\Environment;

class ProcessoService {
    /** @var ProcessoRepository */
    protected $processoRepository;

    /** @var Environment */
    protected $environment;

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(
        ProcessoRepository  $processoRepository,
        Environment $environment,
        EntityManager $entityManager
    ) {
        $this->processoRepository = $processoRepository;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }

    public function getStatusProcesso(): array {
        return $this->processoRepository->getStatusProcesso(
            Utils::getParamters()->getNrorg(),
            Utils::getParamters()->getNrorgPadrao()
        );
    }

    public function getProcessosGestor(
        int $nrParcNegocio
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $processos = $this->processoRepository->getProcessosGestor(
            $nrOrg, $nrParcNegocio
        );

        return $processos;
    }

    public function getFasesTemplate(
        int $nrTemplate
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();

        $fases = $this->processoRepository->getFasesTemplate(
            $nrOrg, $nrTemplate
        );

        return $fases;
    }

    public function getCandidatosFase(
        int $nrProcesso,
        int $nrFase
    ): array {
        $nrOrg = (int) $this->environment->getNrOrg();
        $nrOrgPadrao = (int) $this->environment->getNrOrgPadrao();

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRORGPADRAO' => $nrOrgPadrao,
            'P_NRPROCESSO' => $nrProcesso,
            'P_NRFASE' => $nrFase,
        ];

        $candidatos = $this->processoRepository->getCandidatosFase(
            $params
        );

        return $candidatos;
    }

    public function getCandidatosGeral(
        array $row
    ): array {
        $nrOrg = $this->environment->getNrOrg();
        $nrOrgPadrao = $this->environment->getNrOrgPadrao();

        foreach ($row as $key => $item) {
            if ($item === '') {
                $row[$key] = null;
            }
        }

        if (isset($row['NMPESSOA'])) {
            $row['NMPESSOA'] = StringUtil::prepareForLike($row['NMPESSOA']);
        }

        if (isset($row['DSINSTITUICAO'])) {
            $row['DSINSTITUICAO'] = StringUtil::prepareForLike($row['DSINSTITUICAO']);
        }

        if (isset($row['DSCURSO'])) {
            $row['DSCURSO'] = StringUtil::prepareForLike($row['DSCURSO']);
        }

        if (isset($row['DSAREAFORMACAO'])) {
            $row['DSAREAFORMACAO'] = StringUtil::prepareForLike($row['DSAREAFORMACAO']);
        }

        if (isset($row['DSOCUPACAO'])) {
            $row['DSOCUPACAO'] = StringUtil::prepareForLike($row['DSOCUPACAO']);
        }

        $params = [
            'NRORG'                 => $nrOrg,
            'NRORG_PADRAO'          => $nrOrgPadrao,
            'NMPESSOA'              => $row['NMPESSOA'] ?? null,
            'NRPROCESSO'            => $row['NRPROCESSO'] ?? null,
            'EMAIL'                 => $row['EMAIL'] ?? null,
            'DTNASCPESSOA'          => $row['DTNASCPESSOA'] ?? null,
            'IDADE'                 => $row['IDADE'] ?? null,
            'IDADEMINIMA'           => $row['IDADEMINIMA'] ?? null,
            'IDADEMAXIMA'           => $row['IDADEMAXIMA'] ?? null,
            'SALARIOMINIMO'         => $row['SALARIOMINIMO'] ?? null,
            'SALARIOMAXIMO'         => $row['SALARIOMAXIMO'] ?? null,
            'IDSEXOPESSOA'          => $row['IDSEXOPESSOA'] ?? null,
            'IDCADASTROCOMPLETO'    => $row['IDCADASTROCOMPLETO'] ?? null,
            'CDESTACIVIL'           => $row['CDESTACIVIL'] ?? null,
            'NRNACIONALID'          => $row['NRNACIONALID'] ?? null,
            'IDPOSSUIFILHO'         => $row['IDPOSSUIFILHO'] ?? null,
            'DSINSTITUICAO'         => $row['DSINSTITUICAO'] ?? null,
            'NRCURSO'               => $row['NRCURSO'] ?? null,
            'DSCURSO'               => $row['DSCURSO'] ?? null,
            'NRAREAFORMACAO'        => $row['NRAREAFORMACAO'] ?? null,
            'DSAREAFORMACAO'        => $row['DSAREAFORMACAO'] ?? null,
            'NROCUPACAOM'           => $row['NROCUPACAOM'] ?? null,
            'DSOCUPACAO'            => $row['DSOCUPACAO'] ?? null,
            'CDPAIS'                => $row['CDPAIS'] ?? null,
            'SGESTADO'              => $row['SGESTADO'] ?? null,
            'CDMUNICIPIO'           => $row['CDMUNICIPIO'] ?? null,
        ];

        $candidatos = $this->processoRepository->getCandidatosGeral($params);
        return $candidatos;
    }

    public function addCandidatos(
        array $row
    ): void {
        $candidatos = (array) $row['candidatos'];

        $processo = $this->get($row['NRPROCESSO']);
        $nrTemplate = $processo['NRTEMPLATE'];

        $primeiraFase = $this->getPrimeiraFaseProcesso($nrTemplate);
        $nrFase = (int) $primeiraFase['NRFASE'];

        $this->entityManager->beginTransaction();

        foreach ($candidatos as $cand) {
            $participaProcessoRow = [
                'CDOPERADOR' => $row['CDOPERADOR'],
                'NRPARCNEGOCIOCAND' => $cand['NRPARCNEGOCIO'],
                'NRPROCESSO' => $row['NRPROCESSO'],
                'CDSTATUS' => 2,
            ];

            $this->setParticipaProcesso($participaProcessoRow);

            $participaFaseRow = [
                'CDOPERADOR' => $row['CDOPERADOR'],
                'NRPARCNEGOCIOCAND' => $cand['NRPARCNEGOCIO'],
                'NRPROCESSO' => $row['NRPROCESSO'],
                'NRFASE' => $nrFase,
            ];

            $this->setParticipaFase($participaFaseRow);
        }

        $this->entityManager->commit();
    }

    public function setParticipaProcesso(
        array $row,
        $commit = false
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $nrProcParticipa = NovoCodigo::geraCodigo(
            $nrOrg, 'DES_PARTICIPAPROCESSO', 1, 10, true, $commit
        );

        $params = [
            'NRORG' => $nrOrg,
            'CDOPERADOR' => $row['CDOPERADOR'],
            'NRPROCPARTICIPA' => $nrProcParticipa,
            'NRPARCNEGOCIOCAND' => $row['NRPARCNEGOCIOCAND'] ?? null,
            'NRPARCINDICACAO' => $row['NRPARCINDICACAO'] ?? null,
            'NRORIGEM' => $row['NRORIGEM'] ?? null,
            'NRPROCESSO' => $row['NRPROCESSO'] ?? null,
            'CDSTATUS' => $row['CDSTATUS'] ?? null,
            'DSOBSERVACAOCAND' => $row['DSOBSERVACAOCAND'] ?? null,
            'VRPRETENSAOSALARIO' => $row['VRPRETENSAOSALARIO'] ?? null,
            'VRPRETENCAOSALARIO' => $row['VRPRETENCAOSALARIO'] ?? null,
            'VRMEDIAPROCESSO' => $row['VRMEDIAPROCESSO'] ?? null,
            'NRVINCULOM' => $row['NRVINCULOM'] ?? null,
            'IDPARECERFINAL' => $row['IDPARECERFINAL'] ?? null,
        ];

        $this->processoRepository->setParticipaProcesso($params);
    }

    public function setParticipaFase(
        array $row,
        $commit = false
    ): void {
        $nrOrg = (int) $this->environment->getNrOrg();

        $nrParticipaFase = NovoCodigo::geraCodigo(
            $nrOrg, 'DES_PARTICIPAFASE', 1, 10, true, $commit
        );

        $params = [
            'NRPARCTICIPAFASE' => $nrParticipaFase,
            'NRORG' => $nrOrg,
            'CDOPERADOR' => $row['CDOPERADOR'],
            'NRFASE' => $row['NRFASE'] ?? null,
            'NRPARCNEGOCIOCAND' => $row['NRPARCNEGOCIOCAND'] ?? null,
            'NRPROCESSO' => $row['NRPROCESSO'] ?? null,
            'DTINICIO' => $row['DTINICIO'] ?? null,
            'DTFIM' => $row['DTFIM'] ?? null,
            'NRPONTUACAO' => $row['NRPONTUACAO'] ?? null,
            'NRPERCENTUAL' => $row['NRPERCENTUAL'] ?? null,
            'NRAVALIACAO' => $row['NRAVALIACAO'] ?? null,
            'NRDURACAO' => $row['NRDURACAO'] ?? null,
            'NRPESO' => $row['NRPESO'] ?? null,
            'IDOBRIGATORIO' => $row['IDOBRIGATORIO'] ?? null,
            'DSLOCALFASE' => $row['DSLOCALFASE'] ?? null,
            'IDSTATUS' => $row['IDSTATUS'] ?? 2,
            'CDTIPORESU' => $row['CDTIPORESU'] ?? null,
        ];

        $this->processoRepository->setParticipaFase($params);
    }

    public function getPrimeiraFaseProcesso(
        int $nrTemplate
    ) {
        $fases = $this->getFasesTemplate($nrTemplate);
        $primeiraFase = $fases[0] ?? false;

        if (!$primeiraFase) {
            throw new ExpectedException('Esse processo não possui uma primeira fase!');
        }

        return $primeiraFase;
    }

    public function get(
        int $nrProcesso
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();
        $processo = $this->processoRepository->get($nrOrg, $nrProcesso);

        if (!$nrProcesso) {
            throw new ExpectedException('Não foi encontrado processo ' . $nrProcesso);
        }

        return $processo;
    }
}


